//#include <config.h>
#define _GNU_SOURCE 1

#include <spawn.h>

#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

int
main ()
{
  char *argv[2] = { (char *) "callee", NULL };
  char **parent_env;
  size_t len;
  char **child_env;
  posix_spawn_file_actions_t actions;
  bool actions_allocated;
  posix_spawnattr_t attrs;
  bool attrs_allocated;
  int err;
  pid_t child;
  int status;
  int exitstatus;

  setenv ("PATH", "parentdir:/bin:/usr/bin", 1);
  parent_env = environ;

  {
    char **p;
    char **q;
    len = 0;
    for (p = parent_env; *p != NULL; p++)
      len++;

    child_env = (char **) malloc ((len + 1) * sizeof (char *));
    q = child_env;
    for (p = parent_env; *p != NULL; p++)
      {
        *q = *p;
        if (strncmp (*q, "PATH=", 5) == 0)
          *q = (char *) "PATH=childdir:/bin:/usr/bin";
        q++;
      }
    *q = NULL;
  }

  actions_allocated = false;
  attrs_allocated = false;
  if ((err = posix_spawn_file_actions_init (&actions)) != 0
      || (actions_allocated = true,
          (err = posix_spawnattr_init (&attrs)) != 0
          || (attrs_allocated = true,
              (err = posix_spawnp (&child, "callee", &actions, &attrs, argv, child_env)) != 0)))
    {
      if (actions_allocated)
        posix_spawn_file_actions_destroy (&actions);
      if (attrs_allocated)
        posix_spawnattr_destroy (&attrs);
      errno = err;
      perror ("subprocess failed");
      exit (1);
    }
  posix_spawn_file_actions_destroy (&actions);
  posix_spawnattr_destroy (&attrs);
  status = 0;
  while (waitpid (child, &status, 0) != child)
    ;
  if (!WIFEXITED (status))
    {
      fprintf (stderr, "subprocess terminated with unexpected wait status %d\n", status);
      exit (1);
    }
  exitstatus = WEXITSTATUS (status);
  if (exitstatus != 0)
    {
      fprintf (stderr, "subprocess terminated with unexpected exit status %d\n", exitstatus);
      exit (1);
    }
  return 0;
}
